;; if set to false, optimum models are not prRealed out
(set-option:produce-models true)
(set-option:produce-assignments true)

(declare-fun x1 () Int)
(declare-fun x2 () Int)
(declare-fun x3 () Int)
(declare-fun x4 () Int)
(declare-fun x5 () Real)
(declare-fun x6 () Real)

(assert (<= 2 (+ (* 1 x1) (* 2 x2))))
(assert (<= 3 (+ (* 2 x2) (* 1 x3))))

; Minimization Works:

(push 1)
(minimize (+ (* 2 (+ x1 x2)) (+ x3 x4)) :local-lb 0 :local-ub 100 )
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

(push 1)
(assert (<= 4 (+ (* 1 x3) (* 2 x4))))
(assert (<= 5 (+ (* 2 x4) (* 1 x1))))
(push 1)
(assert (<= 1.1 x1))
(assert (<= 1.1 x2))
(assert (<= 1.1 x3))
(assert (<= 1.1 x4))

;; Minimization Works:
(push 1)
(minimize (+ (* 3 (+ x1 x2)) (+ x3 x4)) :local-lb 0 :local-ub 100 )
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)
(pop 1)
(pop 1)

;; Maximization Works:
(push 1)
(maximize (* 2 (- x1 x2)) :local-lb 3 :local-ub 17 )
(check-sat)
;(set-model -1)
;(get-value (x1))
;(get-value (x2))
;(get-value (x3))
;(get-value (x4))
;(get-value (x5))
;(get-value (x6))
(pop 1)

;; Multiple Objectives work:
;; (first has solution, second has no solution)
;; NOTE: might be solved out of order!
(push 1)
(minimize (+ x1 x2 x3 x4) :local-lb 0 :local-ub 20 )
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

(push 1)
(minimize (+ x1 x2 x3 x4) :local-lb -30 :local-ub -1 )

(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

;; MATHSAT: standard behaviour consistenly conserved

(check-sat)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))

(assert (<= 1000 x1 ))

(check-sat)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))


;; 1. Non-Termination (LAZ): CF is -INF, no early detection in LA(R) solver!
;; STATUS: solved
;; NOTE: -INF detection is based on paper "Recognizing Unbounded Integer Programs"
;;     by R. H. Byrd, A. J. Goldman, Miriam Heller 
(push 1)
(minimize (+ x1 x2))
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

;; 2. Crash (LAR): CF is -INF && not loaded in TABLEAU (no bounds!)
;; STATUS: solved
(push 1)
(minimize (+ x5 x6))
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

;; note: works if lower bounds are pushed
(assert (<= 2 x5))
(assert (<= 2 x6))
(push 1)
(minimize (+ x5 x6))
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

(assert (< 2 x5))
(assert (< 2 x6))
(push 1)
(minimize (+ x5 x6))
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

;; 3. Crash (LAR+LAZ): hybrid CF not yet supported:
;; STATUS: solved
;; NOTE: minimization is done on LAR solver only when entire problem has no mixed_atoms.
;;     (no universal enclosure)
(push 1)
(minimize (+ x1 x2 x3 x4 x5 x6 ))
(check-sat)
(set-model -1)
(get-value (x1))
(get-value (x2))
(get-value (x3))
(get-value (x4))
(get-value (x5))
(get-value (x6))
(pop 1)

;; push/pop
;; STATUS: seems to work
;; push/pop with good example

;; OPEN ISSUES:
;; remove extra assertion?
(check-sat)
(get-model)
